#WBL 20 Feb 2007 $Revision: 1.31 $

#WBL  6 Nov 2007 Save range of data used
#WBL 30 Oct 2008 replace @exprs by exprs()

library(affy);

unitmean=function(exprs) {
  m <- mean(exprs);
# print(m);
  exprs/m;
}
unitmedian=function(exprs) {
  m <- median(exprs);
# print(m);
  exprs/m;
}
unitlmean=function(exprs) {
  m <- exp(mean(log(exprs)));
# print(m);
  exprs/m;
}
unitl90mean=function(exprs) {
  s = sort(exprs);
  m<- exp(mean(log(s[(0.05*length(s)):(0.95*length(s))])));
# print(paste("unitl90mean",m));
  exprs/m;
}

sortbyrow=function(mat) {
# t(apply(t(mat),2,sort));
  apply(mat,2,sort);
}

quantilebyrow=function(mat,rank) {
#rather than sort, use rank to ensure two probes with the same value
#before normalisation, have the same value after normalisation.
  r = apply(mat,2,rank,ties.method="min");
  ans=array(NA,dim=c(nrow(rank),ncol(rank)));
  for(j in seq(from=1,to=ncol(r))) { ans[,j]=rank[r[,j],j]; }
  return(ans);
}
quantile=function(exprs,rankmean) {
# s = sort(exprs,index=TRUE);
# rankmean[s$ix];
#rather than sort, use rank to ensure two probes with the same value
#before normalisation, have the same value after normalisation.
  rankmean[rank(exprs,ties.method="min")];
}

#Global Sorted, avoid huge garbage collection
bubblesort=function(start,end) { #a must be pre-sorted except a[,start]
Mx = (end>start); if(Mx) {E = end-1;} else {E=end+1;}
  d=dim(Sorted);
  w=c(1:d[2]);
  for(i in start:E) {
    if(Mx){j = i+1; w = w[which(Sorted[i,w]>Sorted[j,w])]; }
    else  {j = i-1; w = w[which(Sorted[i,w]<Sorted[j,w])]; }
    t            =  Sorted[i,w];
    Sorted[i,w] <<- Sorted[j,w];
    Sorted[j,w] <<- t;
  }
}


average_all_cel = function(all,file) {

noutliers = round((99+length(all))/200,0);
#noutliers = round((9+length(all))/20,0);
mid       = noutliers+1
if(2*noutliers>=length(all)) {noutliers = 0; mid=0;}
cat(paste("noutliers=",noutliers," mid=",mid,"\n",sep=""));

files = "";
F=0; Files=array(NA,dim=length(all));
for(f in 1:length(all)) {
  cat("all[",f,"] ",all[f],"\n",sep="");
  A <- ReadAffy(filenames=all[f],sd=FALSE);
  if(min(exprs(A))<1 || length(which(is.na(exprs(A))))>0) {
      print(paste("Warning",length(which(exprs(A)<1)),"less than 1",
                  length(which(is.na(exprs(A)))),"is.na in",f,all[f]));
  } else {
  a <- log(exprs(A));
  cat(paste("a[3001] ",a[3001]),"\n");
  if(F==0) {first <- a;} 
  a <- a-first;
  if(F==0) {
    type  <- A@annotation;
    count <- 1;
    S<-sqrt(length(exprs(A)));
    total <- a;
    sum2  <- total^2;
    if(mid>0) {
    Sorted <<- array(NA,dim=c(1+noutliers*2,length(a)));
    Sorted[1,]               <<- a; #min
    Sorted[(1+noutliers*2),] <<- a; #max
    }
  }
  else if(type!=A@annotation) {
    print(paste("ERROR type mismatch",type,f,all[f],A@annotation));
#   exit(0);
  }
  else if(length(total)!=length(exprs(A))) {
    print(paste("ERROR length mismatch",length(total),f,all[f],length(exprs(A))));
#   exit(0);
  }
  else {
    count <- count+1;
    total <- total+a;
    sum2  <- sum2 +(a^2);
    if(mid>0) {
      if(f<mid) {
	Sorted[f,]                 <<- a; #min
	Sorted[(2+noutliers*2-f),] <<- a; #max
	bubblesort(f,1);
	bubblesort(2+noutliers*2-f,1+noutliers*2);
      } else {
	Sorted[mid,] <<- a; 
	bubblesort(mid,1);
	Sorted[mid,] <<- a; #prevent min polluting max
	bubblesort(mid,1+noutliers*2);
      }
    }
  }
  files <- paste(files,all[f]);
  F <- F+1; Files[F] <- all[f];
  }#f ok
}

#reject lowest and highest then calculate geometric mean
if(noutliers>0) { 
Sorted[(noutliers+1),] <<- 0; 
sSorted  = apply(Sorted,2,sum);
sSorted2 = apply(Sorted^2,2,sum);
mean =   (total-sSorted )/(count-2*noutliers);
var  =   (sum2 -sSorted2)/(count-2*noutliers) - mean^2; #log-affy not negative
if(min(var)<0) {
  cat("ERROR negative variance\n");
  cat(paste(which(var<0)));
  cat("\n");
  exit(0);
}
} else        { 
mean =    total/count;
var  =    (sum2/count) - mean^2
}
mean=mean + first;
mean=array(mean,dim=c(S,S));
sd  =array(sqrt(var),dim=c(S,S));

cat(paste(mean[3001],var[3001],sd[3001]),"\n");
#warn = exp(mean + sd*7);
#for(f in 1:length(all)) {
#  a <- ReadAffy(filenames=all[f],sd=FALSE);
#  a <- array(data=a@exprs,dim=c(S,S));
# w <- length(which(a>warn));
# if(w>0) print(paste(type,f,all[f],w,"probes exceed 7sd"));
#


if(file!="") {
write(paste("#average.R mean $Revision: 1.31 $",
     length(mean),date(),type,count,files),
     file=file);
write(mean, file=file, ncolumns=1,append=TRUE);
write(paste("#average.R var $Revision: 1.31 $",
     length(var),date(),type,count,files),
     file=paste(file,"var",sep=""));
write(var, file=paste(file,"var",sep=""), ncolumns=1,append=TRUE);
write(paste("#average.R min $Revision: 1.31 $",
     length(Sorted[noutliers,]),date(),type,count,files),
     file=paste(file,"min",sep=""));
write(Sorted[noutliers,]+first,
     file=paste(file,"min",sep=""),ncolumns=1,append=TRUE);
write(paste("#average.R max $Revision: 1.31 $",
     length(Sorted[(noutliers+2),]),date(),type,count,files),
     file=paste(file,"max",sep=""));
write(Sorted[(noutliers+2),]+first,
     file=paste(file,"max",sep=""), ncolumns=1,append=TRUE);
}
list(annotation=type,lgmean=mean,lgsd=sd,files=array(Files,dim=F));
}
