#$Revision: 1.8 $
#WBL 3 Apr 2007 add remaining idf flaw detection from wbl_idf_flaws.R
#WBL 3 Mar 2007 make into function 

#Look for non-random patterns of trues in S*S Boolean array
#In Affymetrix arrays expect many cases of alternating rows of related
#PM and MM so look only at checker board pattern of neighbours
#G.J.G.Upton,Bioinformatics,21(22) 2005 p4162-4168.

#For efficiency use global C* matrices to form checker board pattern

#cmat S/3 by S/3 id of the microarray that provides the largest of the
#R values of the centre cell.
#  Since we dont have replicates. "The microarray" here means either this
#one or the background reading. For efficiency cmat is coded as 0 and 1
#rather than 1 or 2.

#nmat S/3 by S/3 count of the number of corner cells whose maximum
#values emanates from microarray C.
#  Again we dont have replicates, so "microarray C" means either this
#array or the background. Eg if centre is greater than background, nmat
#is the number of corners (1..4) which also exceed their background.

#pid       number of corners to agree with centre
#threshold number of 8 adjoining 3x3 chequerboard to be suspicious
#          (ie have reached pid) and to agree with central 3x3

idf_flaws_init = function(dd2) {
  #map from S,S to dd2,dd2 Cc centre, Cm left/lower, Cp right/higher
  Cc <<- array(0,dim=dd2); for(q in seq(2,(dd-1),by=3)) Cc[(1+q)/3] <<- q;
  Cm <<- array(0,dim=dd2); for(q in seq(2,(dd-1),by=3)) Cm[(1+q)/3] <<- q-1;
  Cp <<- array(0,dim=dd2); for(q in seq(2,(dd-1),by=3)) Cp[(1+q)/3] <<- q+1;
  #globals for idf_flaws dd2,dd2 neighbourhood CM left/lower, CP right/higher
  CM <<- array(NA,dim=dd2); for(q in 2: dd2   ) CM[q] <<- q-1;
  CP <<- array(NA,dim=dd2); for(q in 1:(dd2-1)) CP[q] <<- q+1;
}

N_matrix = function(ids){
(ids[Cm,Cm]==ids[Cc,Cc])+(ids[Cm,Cp]==ids[Cc,Cc])+
(ids[Cp,Cm]==ids[Cc,Cc])+(ids[Cp,Cp]==ids[Cc,Cc])
}

idf_flaws2 = function(pid,threshold,nmat_max,cmat_max) {#use for both maxima and minima

newm=(nmat_max>=pid)*cmat_max;

t = (newm[CM,CM]==newm); t[is.na(t)]=0; inde_=t;
t = (newm[CM,  ]==newm); t[is.na(t)]=0; inde_=inde_+t;
t = (newm[CM,CP]==newm); t[is.na(t)]=0; inde_=inde_+t;
t = (newm[  ,CM]==newm); t[is.na(t)]=0; inde_=inde_+t;
t = (newm[  ,CP]==newm); t[is.na(t)]=0; inde_=inde_+t;
t = (newm[CP,CM]==newm); t[is.na(t)]=0; inde_=inde_+t;
t = (newm[CP,  ]==newm); t[is.na(t)]=0; inde_=inde_+t;
t = (newm[CP,CP]==newm); t[is.na(t)]=0; inde_=inde_+t;

  (inde_>threshold)*(nmat_max>=pid)*cmat_max;
}#end idf_flaws2

idf_flaws = function(pid,threshold,ids) { #idf SxS boolean 
  idf_flaws2(pid,threshold,N_matrix(ids),ids[Cc,Cc]);
}
